import os
import ahocorasick
from py2neo import Graph


graph = Graph('http://localhost:7474', user='neo4j', password='123456')


category_dict = {
    'city_chinese_name': ['中文', '哪个城市', '哪所城市', '哪座城市', '哪些城市', '城市有哪些'],
    'city_english_name': ['英文'],
    'city_another_name': ['别名', '别称'],
    'province_name': ['省'],
    'province_abbr': ['车牌'],
    'admin_level': ['行政级别'],
    'production': ['生产'],
    'population': ['人口'],
    'neighbor': ['相邻', '毗邻', '接壤'],
}


category_list = [
        'city_chinese_name',
        'city_english_name',
        'city_another_name',
        'province_name',
        'province_abbr',
        'admin_level',
]


mapping = {
    'city_chinese_name': ('城市', 'name'),
    'city_english_name': ('城市', 'englishname'),
    'city_another_name': ('城市', 'anothername'),
    'province_name': ('省份', 'province'),
    'province_abbr': ('城市', 'car'),
    'admin_level': ('行政级别', 'level'),
    'production': ('城市', 'rgdp'),
    'population': ('城市', 'population'),
    'neighbor': ('接壤', 'none'),
}


def is_alphabet(uchar):
    if (u'\u0041' <= uchar <= u'\u005a') or (u'\u0061' <= uchar <= u'\u007a'):
        return True
    else:
        return False


def classify_question(question):
    automation = ahocorasick.Automaton()
    for index, name in enumerate(category_list):
        with open(os.path.join('data', name + '.txt'), 'r', encoding='utf-8')as f:
            for line in f.readlines():
                word = line.strip()
                key = word.lower()
                automation.add_word(key, (index, word))

                # match with city names with or without suffix
                if name == 'city_chinese_name' and key.endswith('市'):
                    automation.add_word(key[:-1], (index, word))
                elif name == 'city_english_name' and key.endswith('city'):
                    automation.add_word(key[:-5], (index, word))

    automation.make_automaton()

    premises = set()
    for end_index, (index, word) in automation.iter(question.lower()):
        # append license plate number when encountering province abbr
        if category_list[index] == 'province_abbr':
            if end_index + 1 < len(question) and is_alphabet(question[end_index + 1]):
                word += question[end_index + 1].upper()
            else:
                continue
        premises.add((category_list[index], word))

    requests = set()
    for item in category_dict.items():
        for word in item[1]:
            if word in question:
                requests.add(item[0])

    keys = set(x[0] for x in premises)
    # filter out another name and english name when chinese name exists
    if 'city_chinese_name' in keys:
        premises = set(filter(lambda x: x[0] != 'city_another_name', premises))
        premises = set(filter(lambda x: x[0] != 'city_english_name', premises))
    # filter out request when the same premise exists
    requests = set(filter(lambda x: x not in keys, requests))

    return premises, requests


def parse_question(question):
    premises, requests = classify_question(question)

    # print(premises)
    # print(requests)

    answers = []
    for (premise, value) in premises:
        mapped_premise = mapping[premise]
        for request in requests:
            mapped_request = mapping[request]

            # print(mapped_premise)
            # print(mapped_request)

            if mapped_premise[0] == mapped_request[0]:
                query = f"match (e:{mapped_premise[0]}) " \
                        f"where e.{mapped_premise[1]}='{value}' " \
                        f"return e.{mapped_request[1]}"
            elif mapped_premise[0] == '城市' and mapped_request[0] == '行政级别':
                query = f"match (a:城市)-[r:属于]->(b:行政级别) " \
                        f"where a.{mapped_premise[1]}='{value}' " \
                        f"return b.level"
            elif mapped_premise[0] == '行政级别' and mapped_request[0] == '城市':
                query = f"match (a:城市)-[r:属于]->(b:行政级别) " \
                        f"where b.{mapped_premise[1]}='{value}' " \
                        f"return a.name"
            elif mapped_premise[0] == '城市' and mapped_request[0] == '省份':
                query = f"match (a:城市)-[r:位于]->(b:省份) " \
                        f"where a.{mapped_premise[1]}='{value}' " \
                        f"return b.province"
            elif mapped_premise[0] == '省份' and mapped_request[0] == '城市':
                query = f"match (a:城市)-[r:位于]->(b:省份) " \
                        f"where b.{mapped_premise[1]}='{value}' " \
                        f"return a.name"
            elif mapped_premise[0] == '城市' and mapped_request[0] == '接壤':
                query = f"match (a:城市)-[r:接壤]->(b:城市) " \
                        f"where a.{mapped_premise[1]}='{value}' " \
                        f"return b.name"
            else:
                query = ''

            # print(query)
            if len(query):
                answers += graph.run(query).data()

    return answers


def main():
    while True:
        question = input('问题：')
        if question == '退出' or question.startswith('q'):
            break
        answers = parse_question(question)
        if len(answers):
            print('答案：')
            for answer in answers:
                for key in answer.keys():
                    print(answer[key])
        else:
            print('抱歉，没有找到答案')


if __name__ == '__main__':
    main()
